/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ce_easyscreenshot[cui-areatype="menu-panel"] > .toolbarbutton-menubutton-dropmarker {
    display: none;
}

#ce_easyscreenshot {
  list-style-image: url(chrome://easyscreenshot-os/skin/icon16.png) !important;
}
#ce_easyscreenshot[cui-areatype="toolbar"]:not([overflowedItem="true"]):-moz-lwtheme-brighttext {
  list-style-image: url(chrome://easyscreenshot-os/skin/icon16-inverted.png) !important;
}
#ce_easyscreenshot[cui-areatype="menu-panel"],
toolbarpaletteitem[place="palette"] > #ce_easyscreenshot {
  list-style-image: url(chrome://easyscreenshot-os/skin/icon32.png) !important;
}

@media (-moz-mac-lion-theme) {
  #ce_easyscreenshot:hover:active:not([disabled="true"]):not([cui-areatype="menu-panel"]) {
    list-style-image: url(chrome://easyscreenshot-os/skin/icon16-active.png) !important;
  }
  #ce_easyscreenshot[cui-areatype="toolbar"]:not([overflowedItem="true"]):hover:active:not([disabled="true"]):-moz-lwtheme-brighttext {
    list-style-image: url(chrome://easyscreenshot-os/skin/icon16-inverted-active.png) !important;
  }
}

@media (-moz-mac-lion-theme) and (min-resolution: 2dppx) {
  #ce_easyscreenshot {
    list-style-image: url(chrome://easyscreenshot-os/skin/icon16@2x.png) !important;
  }
  #ce_easyscreenshot[cui-areatype="toolbar"]:not([overflowedItem="true"]):-moz-lwtheme-brighttext {
    list-style-image: url(chrome://easyscreenshot-os/skin/icon16-inverted@2x.png) !important;
  }
  #ce_easyscreenshot[cui-areatype="menu-panel"],
  toolbarpaletteitem[place="palette"] > #ce_easyscreenshot {
    list-style-image: url(chrome://easyscreenshot-os/skin/icon32@2x.png) !important;
  }

  #ce_easyscreenshot:hover:active:not([disabled="true"]):not([cui-areatype="menu-panel"]) {
    list-style-image: url(chrome://easyscreenshot-os/skin/icon16-active@2x.png) !important;
  }
  #ce_easyscreenshot[cui-areatype="toolbar"]:not([overflowedItem="true"]):hover:active:not([disabled="true"]):-moz-lwtheme-brighttext {
    list-style-image: url(chrome://easyscreenshot-os/skin/icon16-inverted-active@2x.png) !important;
  }

  #ce_easyscreenshot > .toolbarbutton-icon,
  #ce_easyscreenshot > .toolbarbutton-menubutton-button > .toolbarbutton-icon {
    width: 16px;
  }
}
