pref('extensions.easyscreenshot.openDirectory', true);

pref('extensions.easyscreenshot.hotkeys.enabled', false);

pref('extensions.easyscreenshot.hotkeys.screen.select.modifiers', 'control shift');
pref('extensions.easyscreenshot.hotkeys.screen.select.key', 'S');

pref('extensions.easyscreenshot.hotkeys.select.modifiers', 'control shift');
pref('extensions.easyscreenshot.hotkeys.select.key', 'C');

pref('extensions.easyscreenshot.hotkeys.entire.modifiers', 'control shift');
pref('extensions.easyscreenshot.hotkeys.entire.key', 'L');

pref('extensions.easyscreenshot.hotkeys.visible.modifiers', 'control shift');
pref('extensions.easyscreenshot.hotkeys.visible.key', 'V');

pref('extensions.easyscreenshot.homepage', 'chrome://easyscreenshot/locale/easyscreenshot.properties');
