/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * The original code is Abduction!
 * <https://addons.mozilla.org/firefox/addon/abduction/> by M. Evans
 *
 * The MIT License
 *
 * Copyright (c) 2006-09 Rowan Lewis
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

 /**From Abduction!**/

ssSelector-overlay {
	background: transparent;
	cursor: crosshair;
	height: 100%;
	left: 0;
	position: fixed;
	top: 0;
	width: 100%;
	z-index: 2147483647;
}
ssSelector-overlay.x-ray {
	cursor: default;
}
ssSelector-selection,
ssSelector-selection * {
	-moz-box-sizing: border-box;
	display: block;
}
ssSelector-selection {
	background: transparent;
	border: none;
	box-shadow: 0 0 0 100000px rgba(0, 0, 0, 0.6);
	overflow: hidden;
}
ssSelector-btnBox ,
ssSelector-selection {
	position: absolute;
	z-index: 2147483647;
}
ssSelector-selection.x-ray {
	background: hsla(70, 100%, 70%, 0.4);
}
ssSelector-selection-inner {
  background: 0 !important;
	cursor: move;
	height: 100%;
	overflow: hidden;
	width: 100%;
}
ssSelector-selection.x-ray ssSelector-selection-inner {
	display: none;
}
ssSelector-selection-top {
	cursor: n-resize;
	height: 8px;
	left: 8px;
	position: absolute;
	right: 8px;
	top: 0;
}
ssSelector-selection-top-left {
	cursor: nw-resize;
	height: 8px;
	left: 0;
	position: absolute;
	top: 0;
	width: 8px;
}
ssSelector-selection-top-right {
	cursor: ne-resize;
	height: 8px;
	position: absolute;
	right: 0;
	top: 0;
	width: 8px;
}
ssSelector-selection-bottom {
	bottom: 0;
	cursor: s-resize;
	height: 8px;
	left: 8px;
	position: absolute;
	right: 8px;
}
ssSelector-selection-bottom-left {
	bottom: 0;
	cursor: sw-resize;
	height: 8px;
	left: 0;
	position: absolute;
	width: 8px;
}
ssSelector-selection-bottom-right {
	bottom: 0;
	cursor: se-resize;
	height: 8px;
	position: absolute;
	right: 0;
	width: 8px;
}
ssSelector-selection-left {
	bottom: 8px;
	cursor: w-resize;
	left: 0;
	position: absolute;
	top: 8px;
	width: 8px;
}
ssSelector-selection-right {
	bottom: 8px;
	cursor: e-resize;
	position: absolute;
	right: 0;
	top: 8px;
	width: 8px;
}
ssSelector-overlay[state = "selecting"],
ssSelector-selection[state = "selecting"],
ssSelector-selection[state = "selecting"] * {
	cursor: crosshair;
}
ssSelector-overlay[state = "resize-top"],
ssSelector-selection[state = "resize-top"],
ssSelector-selection[state = "resize-top"] * {
	cursor: n-resize;
}
ssSelector-overlay[state = "resize-top-left"],
ssSelector-selection[state = "resize-top-left"],
ssSelector-selection[state = "resize-top-left"] * {
	cursor: nw-resize;
}
ssSelector-overlay[state = "resize-top-right"],
ssSelector-selection[state = "resize-top-right"],
ssSelector-selection[state = "resize-top-right"] * {
	cursor: ne-resize;
}
ssSelector-overlay[state = "resize-bottom"],
ssSelector-selection[state = "resize-bottom"],
ssSelector-selection[state = "resize-bottom"] * {
	cursor: s-resize;
}
ssSelector-overlay[state = "resize-bottom-left"],
ssSelector-selection[state = "resize-bottom-left"],
ssSelector-selection[state = "resize-bottom-left"] * {
	cursor: sw-resize;
}
ssSelector-overlay[state = "resize-bottom-right"],
ssSelector-selection[state = "resize-bottom-right"],
ssSelector-selection[state = "resize-bottom-right"] * {
	cursor: se-resize;
}
ssSelector-overlay[state = "resize-left"],
ssSelector-selection[state = "resize-left"],
ssSelector-selection[state = "resize-left"] * {
	cursor: w-resize;
}
ssSelector-overlay[state = "resize-right"],
ssSelector-selection[state = "resize-right"],
ssSelector-selection[state = "resize-right"] * {
	cursor: e-resize;
}
